@@ -231,6 +231,7 @@ pub enum SyntaxError {
     TS1244,
     TS1245,
     TS1267,
+    TS1273(JsWord),
     TS1383,
     TS2206,
     TS2207,
@@ -578,6 +579,9 @@ impl SyntaxError {
             .into(),
             SyntaxError::TS1245 => "Abstract method cannot have an implementation.".into(),
             SyntaxError::TS1267 => "Abstract property cannot have an initializer.".into(),
+            SyntaxError::TS1273(word) => {
+                format!("'{}' modifier cannot appear on a type parameter", word).into()
+            }
             SyntaxError::TS1383 => "Only named exports may use 'export type'.".into(),
             SyntaxError::TS2206 => "The 'type' modifier cannot be used on a named import when \
                                     'import type' is used on its import statement."
