@@ -178,7 +178,7 @@ fn test262_lexer_error_0002() {
         vec![
             Token::Str {
                 value: "use strict".into(),
-                has_escape: true,
+                raw: "'use\\x20strict'".into(),
             }
             .span(0..15)
             .lb(),
@@ -235,7 +235,6 @@ multiline`"
             Token::Template {
                 cooked: Ok("this\nis\nmultiline".into()),
                 raw: "this\nis\nmultiline".into(),
-                has_escape: false
             },
             tok!('`'),
         ]
@@ -251,7 +250,6 @@ fn tpl_raw_unicode_escape() {
             Token::Template {
                 cooked: Ok(format!("{}", '\u{0010}').into()),
                 raw: "\\u{0010}".into(),
-                has_escape: true
             },
             tok!('`'),
         ]
@@ -276,7 +274,6 @@ fn tpl_invalid_unicode_escape() {
                     ))
                 }),
                 raw: "\\unicode".into(),
-                has_escape: true
             },
             tok!('`'),
         ]
@@ -297,7 +294,6 @@ fn tpl_invalid_unicode_escape() {
                     ))
                 }),
                 raw: "\\u{".into(),
-                has_escape: true
             },
             tok!('`'),
         ]
@@ -318,7 +314,6 @@ fn tpl_invalid_unicode_escape() {
                     ))
                 }),
                 raw: "\\xhex".into(),
-                has_escape: true
             },
             tok!('`'),
         ]
@@ -331,7 +326,7 @@ fn str_escape() {
         lex_tokens(Syntax::default(), r#"'\n'"#),
         vec![Token::Str {
             value: "\n".into(),
-            has_escape: true
+            raw: "'\\n'".into(),
         }]
     );
 }
@@ -342,7 +337,7 @@ fn str_escape_2() {
         lex_tokens(Syntax::default(), r#"'\\n'"#),
         vec![Token::Str {
             value: "\\n".into(),
-            has_escape: true
+            raw: "'\\\\n'".into(),
         }]
     );
 }
@@ -353,7 +348,7 @@ fn str_escape_3() {
         lex_tokens(Syntax::default(), r#"'\x00'"#),
         vec![Token::Str {
             value: "\x00".into(),
-            has_escape: true
+            raw: "'\\x00'".into(),
         }]
     );
 }
@@ -364,7 +359,7 @@ fn str_escape_hex() {
         lex(Syntax::default(), r#"'\x61'"#),
         vec![Token::Str {
             value: "a".into(),
-            has_escape: true,
+            raw: "'\\x61'".into(),
         }
         .span(0..6)
         .lb(),]
@@ -377,7 +372,7 @@ fn str_escape_octal() {
         lex(Syntax::default(), r#"'Hello\012World'"#),
         vec![Token::Str {
             value: "Hello\nWorld".into(),
-            has_escape: true,
+            raw: "'Hello\\012World'".into(),
         }
         .span(0..16)
         .lb(),]
@@ -390,7 +385,7 @@ fn str_escape_unicode_long() {
         lex(Syntax::default(), r#"'\u{00000000034}'"#),
         vec![Token::Str {
             value: "4".into(),
-            has_escape: true,
+            raw: "'\\u{00000000034}'".into(),
         }
         .span(0..17)
         .lb(),]
@@ -705,16 +700,135 @@ fn str_lit() {
         lex_tokens(Syntax::default(), "'abcde'"),
         vec![Token::Str {
             value: "abcde".into(),
-            has_escape: false,
+            raw: "'abcde'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "\"abcde\""),
+        vec![Token::Str {
+            value: "abcde".into(),
+            raw: "\"abcde\"".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'русский'"),
+        vec![Token::Str {
+            value: "русский".into(),
+            raw: "'русский'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\x32'"),
+        vec![Token::Str {
+            value: "2".into(),
+            raw: "'\\x32'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\u1111'"),
+        vec![Token::Str {
+            value: "ᄑ".into(),
+            raw: "'\\u1111'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\u{1111}'"),
+        vec![Token::Str {
+            value: "ᄑ".into(),
+            raw: "'\\u{1111}'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\t'"),
+        vec![Token::Str {
+            value: "\t".into(),
+            raw: "'\t'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\n'"),
+        vec![Token::Str {
+            value: "\n".into(),
+            raw: "'\\n'".into(),
         }],
     );
     assert_eq!(
         lex_tokens(Syntax::default(), "'\\\nabc'"),
         vec![Token::Str {
             value: "abc".into(),
-            has_escape: true,
+            raw: "'\\\nabc'".into(),
         }]
     );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "''"),
+        vec![Token::Str {
+            value: "".into(),
+            raw: "''".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\''"),
+        vec![Token::Str {
+            value: "'".into(),
+            raw: "'\\''".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "\"\""),
+        vec![Token::Str {
+            value: "".into(),
+            raw: "\"\"".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "\"\\\"\""),
+        vec![Token::Str {
+            value: "\"".into(),
+            raw: "\"\\\"\"".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\0'"),
+        vec![Token::Str {
+            value: "\u{0000}".into(),
+            raw: "'\\0'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\n'"),
+        vec![Token::Str {
+            value: "\n".into(),
+            raw: "'\\n'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\r'"),
+        vec![Token::Str {
+            value: "\r".into(),
+            raw: "'\\r'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\012'"),
+        vec![Token::Str {
+            value: "\n".into(),
+            raw: "'\\012'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\07'"),
+        vec![Token::Str {
+            value: "\u{0007}".into(),
+            raw: "'\\07'".into(),
+        }],
+    );
+    assert_eq!(
+        lex_tokens(Syntax::default(), "'\\08'"),
+        vec![Token::Str {
+            value: "\u{0000}8".into(),
+            raw: "'\\08'".into(),
+        }],
+    );
 }
 
 #[test]
@@ -726,7 +840,6 @@ fn tpl_empty() {
             Template {
                 raw: "".into(),
                 cooked: Ok("".into()),
-                has_escape: false
             },
             tok!('`')
         ]
@@ -742,7 +855,6 @@ fn tpl() {
             Template {
                 raw: "".into(),
                 cooked: Ok("".into()),
-                has_escape: false
             },
             tok!("${"),
             Word(Word::Ident("a".into())),
@@ -750,11 +862,29 @@ fn tpl() {
             Template {
                 raw: "".into(),
                 cooked: Ok("".into()),
-                has_escape: false
             },
             tok!('`'),
         ]
-    )
+    );
+
+    assert_eq!(
+        lex_tokens(Syntax::default(), r#"`te\nst${a}test`"#),
+        vec![
+            tok!('`'),
+            Template {
+                raw: "te\\nst".into(),
+                cooked: Ok("te\nst".into()),
+            },
+            tok!("${"),
+            Word(Word::Ident("a".into())),
+            tok!('}'),
+            Template {
+                raw: "test".into(),
+                cooked: Ok("test".into()),
+            },
+            tok!('`'),
+        ]
+    );
 }
 
 #[test]
@@ -919,7 +1049,6 @@ fn issue_191() {
             Token::Template {
                 raw: "".into(),
                 cooked: Ok("".into()),
-                has_escape: false,
             },
             tok!("${"),
             Token::Word(Word::Ident("foo".into())),
@@ -927,7 +1056,6 @@ fn issue_191() {
             Token::Template {
                 raw: "<bar>".into(),
                 cooked: Ok("<bar>".into()),
-                has_escape: false,
             },
             tok!('`')
         ]
@@ -979,7 +1107,7 @@ fn issue_299_01() {
             tok!('='),
             Token::Str {
                 value: "\\ ".into(),
-                has_escape: true
+                raw: "'\\ '".into(),
             },
             Token::JSXTagEnd,
             JSXText { raw: "ABC".into() },
@@ -1013,7 +1141,313 @@ fn issue_299_02() {
             tok!('='),
             Token::Str {
                 value: "\\\\".into(),
-                has_escape: true
+                raw: "'\\\\'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_1() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='abc'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "abc".into(),
+                raw: "'abc'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_2() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num=\"abc\">ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "abc".into(),
+                raw: "\"abc\"".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_3() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='\n'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "\n".into(),
+                raw: "'\n'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_4() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='&sup3;'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "³".into(),
+                raw: "'&sup3;'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_5() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='&#42;'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "*".into(),
+                raw: "'&#42;'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_6() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='&#x23;'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "#".into(),
+                raw: "'&#x23;'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_7() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='&'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "&".into(),
+                raw: "'&'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_8() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='&;'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "&;".into(),
+                raw: "'&;'".into(),
+            },
+            Token::JSXTagEnd,
+            JSXText { raw: "ABC".into() },
+            JSXTagStart,
+            tok!('/'),
+            JSXName {
+                name: "Page".into()
+            },
+            JSXTagEnd,
+            Semi,
+        ]
+    );
+}
+
+#[test]
+fn jsx_string_9() {
+    assert_eq!(
+        lex_tokens(
+            crate::Syntax::Es(crate::EsConfig {
+                jsx: true,
+                ..Default::default()
+            }),
+            "<Page num='&&'>ABC</Page>;"
+        ),
+        vec![
+            Token::JSXTagStart,
+            Token::JSXName {
+                name: "Page".into()
+            },
+            Token::JSXName { name: "num".into() },
+            tok!('='),
+            Token::Str {
+                value: "&&".into(),
+                raw: "'&&'".into(),
             },
             Token::JSXTagEnd,
             JSXText { raw: "ABC".into() },
@@ -1034,7 +1468,7 @@ fn issue_316() {
         lex_tokens(Default::default(), "'Hi\\r\\n..'"),
         vec![Token::Str {
             value: "Hi\r\n..".into(),
-            has_escape: true
+            raw: "'Hi\\r\\n..'".into(),
         }]
     );
 }
@@ -1046,7 +1480,7 @@ fn issue_401() {
         vec![
             Token::Str {
                 value: "17".into(),
-                has_escape: false
+                raw: "'17'".into(),
             },
             tok!("as"),
             tok!("const")
@@ -1098,7 +1532,7 @@ fn issue_915_1() {
             LParen,
             Token::Str {
                 value: "\r\n".into(),
-                has_escape: true
+                raw: "\"\\r\\n\"".into(),
             },
             RParen
         ]
@@ -1329,7 +1763,7 @@ fn issue_2853_1_js() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: true,
+                raw: "\"\\0a\"".into(),
             }
         ],
     );
@@ -1351,7 +1785,7 @@ fn issue_2853_2_ts() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: true,
+                raw: "\"\\0a\"".into(),
             }
         ],
     );
@@ -1373,7 +1807,7 @@ fn issue_2853_3_js() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: false,
+                raw: "\"\u{0000}a\"".into(),
             }
         ],
     );
@@ -1395,7 +1829,7 @@ fn issue_2853_4_ts() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: false,
+                raw: "\"\u{0000}a\"".into(),
             }
         ],
     );
@@ -1420,7 +1854,7 @@ fn issue_2853_5_jsx() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: true,
+                raw: "\"\\0a\"".into(),
             }
         ]
     );
@@ -1445,7 +1879,7 @@ fn issue_2853_6_tsx() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: true,
+                raw: "\"\\0a\"".into(),
             }
         ]
     );
@@ -1470,7 +1904,7 @@ fn issue_2853_7_jsx() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: false,
+                raw: "\"\u{0000}a\"".into(),
             }
         ]
     );
@@ -1495,7 +1929,7 @@ fn issue_2853_8_tsx() {
             Token::AssignOp(AssignOpToken::Assign),
             Token::Str {
                 value: "\u{0000}a".into(),
-                has_escape: false,
+                raw: "\"\u{0000}a\"".into(),
             }
         ]
     );
